@@ -1,8 +1,15 @@
-from typing import Union, overload
+from functools import singledispatch
+from typing import overload, Union
 
 import numba
 import numpy as np
-import pandas as pd
+
+try:
+    import pandas as pd
+
+    pandas_import = True
+except ModuleNotFoundError:
+    pandas_import = False
 
 
 @numba.jit(nopython=True, fastmath=True, cache=True)
@@ -67,35 +74,59 @@ def _quantile_normalize(_in_arr: np.ndarray) -> np.ndarray:
     return qnorm
 
 
-# fmt: off
-# function overloading for the correct return type depending on the input
-@overload
-def quantile_normalize(data: pd.DataFrame) -> pd.DataFrame: ...
-@overload
-def quantile_normalize(data: np.ndarray) -> np.ndarray: ...
-# fmt: on
+if pandas_import:
+    # fmt: off
+    # function overloading for the correct return type depending on the input
+    @overload
+    def quantile_normalize(data: pd.DataFrame) -> pd.DataFrame: ...
+    @overload
+    def quantile_normalize(data: np.ndarray) -> np.ndarray: ...
+    # fmt: on
+
+    @singledispatch
+    def quantile_normalize(
+        data: Union[pd.DataFrame, np.ndarray]
+    ) -> Union[pd.DataFrame, np.ndarray]:
+        """
+        Quantile normalize your array/dataframe.
+
+        returns: a quantile normalized copy of the input.
+        """
+        raise NotImplementedError(
+            f"quantile_normalize not implemented for type {type(data)}"
+        )
 
 
-def quantile_normalize(
-    data: Union[pd.DataFrame, np.ndarray]
-) -> Union[pd.DataFrame, np.ndarray]:
-    """
-    Quantile normalize your array/dataframe.
+else:
 
-    returns: a quantile normalized copy of the input.
-    """
-    if not isinstance(data, (pd.DataFrame, np.ndarray)):
-        raise NotImplementedError
+    @singledispatch
+    def quantile_normalize(data: np.ndarray) -> np.ndarray:
+        """
+        Quantile normalize your array.
 
-    if len(data.shape) != 2:
-        raise ValueError
+        returns: a quantile normalized copy of the input.
+        """
+        raise NotImplementedError(
+            f"quantile_normalize not implemented for type {type(data)}"
+        )
 
-    if isinstance(data, pd.DataFrame):
+
+@quantile_normalize.register(np.ndarray)
+def quantile_normalize_np(data: np.ndarray) -> np.ndarray:
+    if data.dtype not in [np.float32, np.float64]:
+        raise ValueError(
+            f"The type of your data ({data.dtype}) is is not supported, "
+            f"and might lead to undefined behaviour. Please convert to float32"
+            f" or float64. "
+        )
+
+    return _quantile_normalize(data)
+
+
+if pandas_import:
+
+    @quantile_normalize.register(pd.DataFrame)
+    def quantile_normalize_pd(data: pd.DataFrame) -> pd.DataFrame:
         qn_data = data.copy()
-        qn_data[:] = _quantile_normalize(qn_data.values)
-    elif isinstance(data, np.ndarray):
-        qn_data = _quantile_normalize(data)
-    else:
-        raise NotImplementedError
-
-    return qn_data
+        qn_data[:] = quantile_normalize_np(qn_data.values)
+        return qn_data
